import time
import sys
import Gpib
import signal
import numbers 

temp_set = 15.0
temp_sample = 15
vset = 0.0
result = []
sample = 0

SOAK_TIME = 10 * 1 # 5 min soak time

# Add SMU gpib code
# Add TEC SMU gpib code
# add file handling and generate DSV file

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()  

def set_temp(pv):
    print ("Programming temperature process value %5.3f C" % pv)
    time.sleep(1)

def soak(sec):
    print ("Soaking for %3.2f minutes..." % (sec / 60))
    time.sleep(sec)
    
def meas_current():
    kei2400.write (":MEAS:CURR:DC?")
    kdata = kei2400.read() 
    dat = kdata.split(",")
    return float(dat[1])

def meas_voltage():
    kei2400.write (":MEAS:VOLT:DC?")
    kdata = kei2400.read() 
    dat = kdata.split(",")
    return float(dat[0])

def prog_volt(voltage):
    kei2400.write (":SOUR:VOLT %f" % voltage) 
    
    
kei2400_idn = "KEITHLEY INSTRUMENTS INC."
kei2400_mdl = "MODEL 2400" 
kei2400 = Gpib.Gpib(0,24)
kei2400.clear()
sys.stdout.write("\033[0;36m Keithley 2400 I-V xDevs.com test tool \r\n  Using NI GPIB adapter with next instruments config: \r\n * GPIB 24 : Keithley 2400 SMU \r\n ! Do not swap terminals during calibration, high voltage may be present !\033[0;39m\r\n\r\n")

kei2400.write ("*IDN?")
idn_str = kei2400.read()
idmfg = idn_str.split(",")
if (kei2400_idn == idmfg[0]):
    if (kei2400_mdl == idmfg[1]):
        sys.stdout.write("\033[0;32mKeithley Model 2400 - detected, S/N %s, Version: %s\033[0;39m\r\n" % (idmfg[2], idmfg[3]))
    else:
        print ("\033[0;31mIncorrect Model! Check GPIB address. Calibration cancelled.\033[0;39m")    
        quit()
else:
    sys.stdout.write ("\033[0;31mNo Keithley instrument detected. Check GPIB address. Calibration cancelled.\033[0;39m")
    quit()

kei2400.write ("*CLS")
kei2400.write ("*ESE 1;*SRE 32")
kei2400.write ("*RST")
kei2400.write (":SENS:FUNC:CONC OFF")
kei2400.write (":SENS:FUNC 'VOLT:DC'")
kei2400.write (":SOUR:FUNC VOLT")
kei2400.write (":OUTP:STAT OFF")

vset = -10.0
#Set SMU to source voltage, measure current 100mA range
kei2400.write (":SOUR:FUNC VOLT")
kei2400.write (":SENS:CURR:PROT 0.02")            # Icmpl = 20mA
kei2400.write (":SENS:CURR:RANG 0.1")
kei2400.write (":SOUR:VOLT %3.1f" % vset) 
kei2400.write (":SOUR:VOLT:PROT:LEV MAX")
kei2400.write (":SYST:RSEN OFF")

with open("2DW232-pin1_hi-pin2_lo.csv", 'a') as fou:
    fou.write ("sample;volt;curr;temp;\r")
fou.close

for temp_sample in range (15, 71):
    vset = -10.0
    
    for sample in range (0, 6001):
        curr = meas_current()
        volt = meas_voltage()
        result.append(curr)
	with open("2DW232-1.log", 'a') as fou:
	    fou.write ("%d;%8.6f;%e;%5.3f;\r" % (sample, volt, curr, temp_set))
        print ("Sample: %d, Vsour: %8.6f V, Imeas: %e A, Temp = %5.3f C" % (sample, volt, curr, temp_set))
        vset = vset + 5E-3
	prog_volt(vset)
    fou.close()
    temp_set = temp_set + 1.0
    set_temp(temp_set)
    soak(SOAK_TIME)